module ParamsParser
  module RailsIntegration
    def parse(params)
      permit(super(params))
    end

    private

    def permit(params)
      case params
      when ActionController::Parameters
        # The ParamsParser filters out parameters that are not explicitly
        # configured and #parse creates a new object so we are safe to call
        # #permit! here.
        #
        # 1. Any non configured parameters are already filtered out
        # 2. A new object is created by #parse so we are not mutating and
        #    implicitly permitting the parameters that were passed in.
        params.permit!
      else
        params
      end
    end
  end

  Parser.prepend RailsIntegration
end
